@@ -30,6 +30,7 @@
  */
 
 
+#pragma once
 #ifndef __UPX_P_UNIX_H
 #define __UPX_P_UNIX_H 1
 
@@ -41,6 +42,8 @@
 
 class PackUnix : public Packer
 {
+public:
+    ~PackUnix();
     typedef Packer super;
 protected:
     PackUnix(InputFile *f);
@@ -52,8 +55,8 @@ public:
     virtual void pack(OutputFile *fo) override;
     virtual void unpack(OutputFile *fo) override;
 
-    virtual bool canPack() override;
-    virtual int  canUnpack() override; // bool, except -1: format known, but not packed
+    virtual tribool canPack() override;
+    virtual tribool canUnpack() override; // bool, except -1: format known, but not packed
     int find_overlay_offset(MemBuffer const &buf);
 
 protected:
@@ -82,12 +85,12 @@ protected:
         bool inhibit_compression_check = false);
     virtual unsigned unpackExtent(unsigned wanted, OutputFile *fo,
         unsigned &c_adler, unsigned &u_adler,
-        bool first_PF_X, unsigned szb_info,
+        bool first_PF_X,
         int is_rewrite = false  // 0(false): write; 1(true): rewrite; -1: no write
         );
     unsigned total_in, total_out;  // unpack
 
-    int exetype;
+    int exetype;  // 0: unknown; 1: ELF; 2: pre-ELF; -1: /bin/sh; -2: Java
     unsigned blocksize;
     unsigned progid;              // program id
     unsigned overlay_offset;      // used when decompressing
@@ -99,30 +102,32 @@ protected:
 
     unsigned b_len;  // total length of b_info blocks
     unsigned methods_used;  // bitmask of compression methods
+    unsigned szb_info;  // 3*4 (sizeof b_info); or 2*4 if ancient
+    unsigned saved_opt_android_shlib;
 
     // must agree with stub/linux.hh
-    __packed_struct(b_info) // 12-byte header before each compressed block
+    packed_struct(b_info) { // 12-byte header before each compressed block
         NE32 sz_unc;  // uncompressed_size
         NE32 sz_cpr;  //   compressed_size
         unsigned char b_method;  // compression algorithm
         unsigned char b_ftid;  // filter id
         unsigned char b_cto8;  // filter parameter
         unsigned char b_extra;
-    __packed_struct_end()
+    };
 
-    __packed_struct(l_info) // 12-byte trailer in header for loader
+    packed_struct(l_info) { // 12-byte trailer in header for loader
         LE32 l_checksum;
         LE32 l_magic;
         LE16 l_lsize;
         unsigned char l_version;
         unsigned char l_format;
-    __packed_struct_end()
+    };
 
-    __packed_struct(p_info) // 12-byte packed program header
+    packed_struct(p_info) { // 12-byte packed program header
         NE32 p_progid;
         NE32 p_filesize;
         NE32 p_blocksize;
-    __packed_struct_end()
+    };
 
     struct l_info linfo;
 
@@ -135,6 +140,7 @@ protected:
 // abstract classes encapsulating endian issues
 // note: UPX_MAGIC is always stored in le32 format
 **************************************************************************/
+
 class PackUnixBe32 : public PackUnix
 {
     typedef PackUnix super;
@@ -142,28 +148,28 @@ protected:
     PackUnixBe32(InputFile *f) : super(f) { bele = &N_BELE_RTP::be_policy; }
 
     // must agree with stub/linux.hh
-    __packed_struct(b_info) // 12-byte header before each compressed block
+    packed_struct(b_info) { // 12-byte header before each compressed block
         BE32 sz_unc;  // uncompressed_size
         BE32 sz_cpr;  //   compressed_size
         unsigned char b_method;  // compression algorithm
         unsigned char b_ftid;  // filter id
         unsigned char b_cto8;  // filter parameter
         unsigned char b_extra;
-    __packed_struct_end()
+    };
 
-    __packed_struct(l_info) // 12-byte trailer in header for loader
+    packed_struct(l_info) { // 12-byte trailer in header for loader
         BE32 l_checksum;
         BE32 l_magic;
         BE16 l_lsize;
         unsigned char l_version;
         unsigned char l_format;
-    __packed_struct_end()
+    };
 
-    __packed_struct(p_info) // 12-byte packed program header
+    packed_struct(p_info)  {// 12-byte packed program header
         BE32 p_progid;
         BE32 p_filesize;
         BE32 p_blocksize;
-    __packed_struct_end()
+    };
 };
 
 
@@ -174,28 +180,28 @@ protected:
     PackUnixLe32(InputFile *f) : super(f) { bele = &N_BELE_RTP::le_policy; }
 
     // must agree with stub/linux.hh
-    __packed_struct(b_info) // 12-byte header before each compressed block
+    packed_struct(b_info) { // 12-byte header before each compressed block
         LE32 sz_unc;  // uncompressed_size
         LE32 sz_cpr;  //   compressed_size
         unsigned char b_method;  // compression algorithm
         unsigned char b_ftid;  // filter id
         unsigned char b_cto8;  // filter parameter
         unsigned char b_extra;
-    __packed_struct_end()
+    };
 
-    __packed_struct(l_info) // 12-byte trailer in header for loader
+    packed_struct(l_info) { // 12-byte trailer in header for loader
         LE32 l_checksum;
         LE32 l_magic;
         LE16 l_lsize;
         unsigned char l_version;
         unsigned char l_format;
-    __packed_struct_end()
+    };
 
-    __packed_struct(p_info) // 12-byte packed program header
+    packed_struct(p_info) { // 12-byte packed program header
         LE32 p_progid;
         LE32 p_filesize;
         LE32 p_blocksize;
-    __packed_struct_end()
+    };
 };
 
 
